<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xdt="http://www.w3.org/2005/02/xpath-datatypes"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:m="http//www.mulberrytech.com/xslt/util"
  exclude-result-prefixes="xdt xs m">
  
  <xsl:output indent="yes"/>
  
  <!-- start with:

  java -jar ../bin/saxon8.jar timesheet.xml timesheet-layout.xsl start=2006-06-28
  
  extracts seven days from timesheet.xml
  
  phase 1: extract requested dates from log
  phase 2: generate analytical profile of extract as a table (a grouping pass)
  phase 3: display all this nicely
  [phase 4: populate a blank spreadsheet with this data]

  -->
  
  <xsl:param name="start" select="m:get-date(/descendant::date[1])"/>
  <!--<xsl:param name="start" select="min(for $d in //date return m:get-date($d))"/>-->
  <xsl:variable name="start-date" select="xs:date($start)"/>
  
  <xsl:variable name="end-date" select="$start-date + (xdt:dayTimeDuration('P1D') * 6)"/>

  <xsl:variable name="week"
    select="for $d in (0 to 6) return
     ($start-date + (xdt:dayTimeDuration('P1D') * $d))" as="xs:date*"/>
  
  <xsl:variable name="timesheet" select="/"/>
  
  <xsl:variable name="extract">
    <extract>
      <xsl:apply-templates select="//day[m:get-date(date) = $week]" mode="extract"/>
    </extract>
  </xsl:variable>

  <xsl:template match="*|@*" mode="extract">
    <xsl:copy>
      <xsl:apply-templates select="@* | node()" mode="extract"/>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="date" mode="extract">
    <date value="{m:infer-date(.)}">
      <!-- note if the date already has a value, it overrides the inferred value -->
      <xsl:apply-templates select="@* | node()" mode="extract"/>
    </date>
  </xsl:template>

  <xsl:template match="/">
    <!--<xsl:copy-of select="$extract"/>-->
    <xsl:apply-templates select="$extract/*"/>
  </xsl:template>
  
  <xsl:template match="extract">
    <timesheet>
      <xsl:copy-of select="$timesheet/timesheet/employee"/>
      <xsl:variable name="days" select="day"/>
      <xsl:for-each-group select="//entry" group-by="account/project">
        <xsl:variable name="project" select="current-grouping-key()"/>
        <xsl:call-template name="merge-entries">
          <xsl:with-param name="days" select="$days"/>
          <xsl:with-param name="project" select="$project"/>
          <xsl:with-param name="job" select="()"/>
          <xsl:with-param name="role" select="()"/>
        </xsl:call-template>
        <xsl:for-each-group select="current-group()" group-by="work/role">
          <xsl:variable name="role" select="current-grouping-key()"/>
          <xsl:call-template name="merge-entries">
            <xsl:with-param name="days" select="$days"/>
            <xsl:with-param name="project" select="$project"/>
            <xsl:with-param name="job" select="()"/>
            <xsl:with-param name="role" select="$role"/>
          </xsl:call-template>
        </xsl:for-each-group>
        <xsl:for-each-group select="current-group()" group-by="account/job">
          <xsl:variable name="job" select="current-grouping-key()"/>
          <xsl:call-template name="merge-entries">
            <xsl:with-param name="days" select="$days"/>
            <xsl:with-param name="project" select="$project"/>
            <xsl:with-param name="job" select="$job"/>
            <xsl:with-param name="role" select="()"/>
          </xsl:call-template>
          <xsl:for-each-group select="current-group()" group-by="work/role">
            <xsl:variable name="role" select="current-grouping-key()"/>
            <xsl:call-template name="merge-entries">
              <xsl:with-param name="days" select="$days"/>
              <xsl:with-param name="project" select="$project"/>
              <xsl:with-param name="job" select="$job"/>
              <xsl:with-param name="role" select="$role"/>
            </xsl:call-template>
          </xsl:for-each-group>
        </xsl:for-each-group>
      </xsl:for-each-group>
    </timesheet>
  </xsl:template>
  
  <xsl:template name="merge-entries">
    <xsl:param name="days" as="element(day)+"/>
    <xsl:param name="project"/>
    <xsl:param name="job"/>
    <xsl:param name="role"/>
    <xsl:variable name="sessions" select="$days/entry[account/project=$project
      and (if (exists($job)) then account/job=$job else not(account/job))
      and (if (exists($role)) then work/role=$role else not(work/role))]"/>
    <xsl:if test="$sessions">
      <account>
        <!-- We emit placeholder elements for all of project, job and role
             even if none is given. -->
        <project>
          <xsl:value-of select="$project"/>
        </project>
        <job>
          <xsl:value-of select="$job"/>
        </job>
        <role>
          <xsl:value-of select="$role"/>
        </role>
        <xsl:for-each select="$week">
          <xsl:variable name="today" select="."/>
          <day date="{$today}">
            <xsl:apply-templates select="$sessions intersect
              $days[date/@value = $today]/entry" mode="filter"/>
          </day>
        </xsl:for-each>
      </account>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="* | @*" mode="filter">
    <xsl:copy>
      <xsl:apply-templates select="node() | @*" mode="filter"/>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="account | text()[not(normalize-space())]" mode="filter"/>

  <xsl:template match="time" mode="filter">
    <xsl:copy>
      <xsl:copy-of select="@*"/>
      <xsl:if test="not(exists(hours))">
        <hours>
          <xsl:value-of select="format-number((m:effort(.) div xdt:dayTimeDuration('PT1H')),'0.##')"/>
        </hours>
      </xsl:if>
      <xsl:apply-templates mode="filter"/>
    </xsl:copy>
    
  </xsl:template>
  <xsl:variable name="any-month"
    select="('January','February','March','April','May','June',
    'July','August','September','October','November','December')"/>
  
  <xsl:function name="m:get-date" as="xs:date">
    <xsl:param name="date" as="element(date)"/>
    <xsl:choose>
      <xsl:when test="$date/@value">
        <xsl:value-of select="xs:date($date/@value)"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="m:infer-date($date)"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:function>
  
  <xsl:function name="m:infer-date" as="xs:date">
    <xsl:param name="date" as="element(date)"/>
    <xsl:variable name="date-fragments"
      select="for $t in (tokenize($date,' ')) return translate($t,',','')"/>
    <xsl:variable name="year"
      select="$date-fragments[number(.) and string-length(.)=4]"/>
    <xsl:variable name="month-as-string"
      select="$date-fragments[. = $any-month]"/>
    <xsl:variable name="month"
      select="format-number(index-of($any-month,$month-as-string),'00')"/>
    <xsl:variable name="day"
      select="format-number(number($date-fragments[number(.) le 31]),'00')"/>
    <xsl:value-of select="string-join(($year,$month,$day),'-')"/>
  </xsl:function>
  
  <xsl:function name="m:effort" as="xdt:dayTimeDuration">
    <xsl:param name="time" as="element(time)"/>
    <xsl:variable name="time-expended" select="if ($time/hours)
      then (number($time/hours) * xdt:dayTimeDuration('PT1H'))
      else m:duration($time)"/>
    <xsl:value-of select="$time-expended"/>
  </xsl:function>
  
  <xsl:function name="m:duration" as="xdt:dayTimeDuration">
    <xsl:param name="time" as="element(time)"/>
    <xsl:variable name="starting" select="m:normalize-time($time/start)"/>
    <xsl:variable name="ending" select="m:normalize-time($time/end)"/>
    <xsl:value-of select="$ending - $starting"/>
  </xsl:function>
  
  <xsl:function name="m:normalize-time" as="xs:time">
    <xsl:param name="moment" as="element()"/>
    <xsl:variable name="nominal-time">
      <xsl:value-of select="format-number((number(substring-before($moment,':')) mod 12),'00')"/>
      <xsl:text>:</xsl:text>
      <xsl:value-of select="translate(substring-after($moment,':'),'apm.','')"/>
      <xsl:text>:00</xsl:text>
    </xsl:variable>
    <xsl:variable name="typed-nominal-time" select="xs:time(string($nominal-time))"/>
    <xsl:value-of select="if (contains($moment,'p')) then 
      ($typed-nominal-time + (xdt:dayTimeDuration('PT12H')))
      else ($typed-nominal-time)"/>
  </xsl:function>

</xsl:stylesheet>
