<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xdt="http://www.w3.org/2005/02/xpath-datatypes"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:m="http://www.mulberrytech.com/XSLT/util" version="2.0">

  <xsl:output method="text"/>

  <xsl:key name="entries-by-project" match="entry" use="account/project"/>

  <xsl:variable name="distinct-projects" select="distinct-values(//project)"/>

  <xsl:template name="test" match="/">
    <xsl:text>distinct-values(//project):&#xA;</xsl:text>
    <xsl:value-of select="$distinct-projects" separator="&#xA;"/>
    
    <xsl:text>&#xA;&#xA;Grouping by distinct values of project and summing their given times&#xA;</xsl:text>
    <xsl:value-of select="for $c in $distinct-projects return (concat($c, ': ',
      sum(key('entries-by-project',$c)/time/hours)))" separator="&#xA;"/>
    
    <xsl:text>&#xA;&#xA;Grouping by distinct values of project and summing their actual times&#xA;</xsl:text>
    <!-- xsl:value-of select="for $c in $distinct-projects return (concat($c, ': ',
      sum(key('entries-by-project',$c)/m:effort(time))))" separator="&#xA;" -->
      <xsl:value-of select="for $c in $distinct-projects return (concat($c, ': ',
      m:count-hours(sum(key('entries-by-project',$c)/m:effort(time)))))" separator="&#xA;"/>
    
  </xsl:template>

  <xsl:function name="m:count-hours" as="xs:double">
    <xsl:param name="duration" as="xdt:dayTimeDuration"/>
    <xsl:variable name="duration-value" select="(24 * days-from-duration($duration)) +
      hours-from-duration($duration) +
      (minutes-from-duration($duration) div 60)"/>
    <xsl:value-of select="round($duration-value * 100) div 100"/>
  </xsl:function>
  
  <xsl:function name="m:effort" as="xdt:dayTimeDuration">
    <xsl:param name="time" as="element(time)"/>
    <xsl:variable name="time-expended" select="if ($time/hours)
      then (number($time/hours) * xdt:dayTimeDuration('PT1H'))
      else m:duration($time)"/>
    <xsl:value-of select="$time-expended"/>
  </xsl:function>
  
  <xsl:function name="m:duration" as="xdt:dayTimeDuration">
    <xsl:param name="time" as="element(time)"/>
    <xsl:variable name="starting" select="m:normalize-time($time/start)"/>
    <xsl:variable name="ending" select="m:normalize-time($time/end)"/>
    <xsl:value-of select="$ending - $starting"/>
  </xsl:function>
  
  <xsl:function name="m:normalize-time" as="xs:time">
    <xsl:param name="moment" as="element()"/>
    <xsl:variable name="nominal-time">
      <xsl:value-of select="format-number((number(substring-before($moment,':')) mod 12),'00')"/>
      <xsl:text>:</xsl:text>
      <xsl:value-of select="translate(substring-after($moment,':'),'apm.','')"/>
      <xsl:text>:00</xsl:text>
    </xsl:variable>
    <xsl:variable name="typed-nominal-time" select="xs:time(string($nominal-time))"/>
    <xsl:value-of select="if (contains($moment,'p')) then 
      ($typed-nominal-time + (xdt:dayTimeDuration('PT12H')))
      else ($typed-nominal-time)"/>
  </xsl:function>
  
</xsl:stylesheet>
