<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0"
  xmlns:xdt="http://www.w3.org/2005/02/xpath-datatypes"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  exclude-result-prefixes="#all">

  <!-- takes as input the output of collate-entries.xsl operating on a timesheet
    to crunch and group it; this is a straightforward presentation -->
  
  <xsl:variable name="logo" select="'file:///c:/Projects/timesheet/v2.0/xslt/smbtree.gif'"/>
  
  <xsl:template match="/">
    <html>
      <head>
        <style type="text/css">

body { background-color: gainsboro } 
tr.even { background-color: white }
tr.odd  { background-color: palegreen }
tr.total { background-color: lightsteelblue }

        </style>
      </head>
      <body>
        <xsl:apply-templates select="timesheet/employee" mode="header"/>
        <h2 style="margin-top:0em">
          <xsl:value-of select="format-date(xs:date((descendant::*/@date)[1]),'[FNn] [MNn] [D], [Y]')"/>
          <xsl:text> - </xsl:text>
          <xsl:value-of select="format-date(xs:date((descendant::*/@date)[last()]),'[FNn] [MNn] [D], [Y]')"/>
        </h2>
        <xsl:call-template name="summary-table"/>
        <xsl:apply-templates mode="report"/>
        <img href="{$logo}"/>
      </body>
    </html>
  </xsl:template>

  <xsl:template match="employee" mode="header">
    <h4 style="margin-bottom:0em; margin-top:0em">
      <xsl:apply-templates/>
    </h4>
  </xsl:template>

  <xsl:template match="employee" mode="report"/>
  <xsl:template match="employee" mode="summary"/>
  
  <xsl:template name="summary-table">
    <table cellspacing="2" cellpadding="5">
      <tr>
        <td>
          <b>project</b>
        </td>
        <td>
          <b>job</b>
        </td>
        <td>
          <b>role</b>
        </td>
        <xsl:for-each select="*/account[1]/day">
          <td style="text-align:center">
            <xsl:value-of
              select="substring(format-date(xs:date(@date),'[FNn]'),1,3)"/>
          </td>
        </xsl:for-each>
        <td>
          <b>hours</b>
        </td>
      </tr>
      <xsl:apply-templates mode="summary"/>
      <tr class="total">
        <td colspan="3">
          <b>Total</b>
        </td>
        <xsl:for-each select="*/account[1]/day">
          <td style="text-align:right">
            <xsl:value-of select="sum(//day[@date=current()/@date]//hours)"/>
          </td>
        </xsl:for-each>
        <td style="text-align:right">
          <b>
            <xsl:value-of select="sum(//hours)"/>
          </b>
        </td>
      </tr>
    </table>
  </xsl:template>

  <xsl:template match="account" mode="summary">
    <tr class="{if (count(preceding-sibling::account) mod 2) then 'even' else 'odd'}">
      <xsl:apply-templates mode="summary"/>
      <td style="text-align:right">
        <xsl:value-of select="sum(.//hours)"/>
      </td>
    </tr>
  </xsl:template>
  
  <xsl:template match="project | job | role" mode="summary">
    <td>
      <xsl:apply-templates/>
      <xsl:if test="not(normalize-space())">&#xA0;</xsl:if>
    </td>
  </xsl:template>
  
  <xsl:template match="day" mode="summary">
    <td style="text-align:right">
      <xsl:value-of select="sum(.//hours)"/>
    </td>
  </xsl:template>
  
  <xsl:template match="account" mode="report">
    <div class="account" style="margin-top:1em; padding: 0.5em;
      background-color: honeydew; border: thin dashed black">
    <h3 style="margin-top: 0em; margin-bottom: 0.5em">
      <xsl:apply-templates select="(project | job | role)[normalize-space()]" mode="mention"/>
    </h3>
    <xsl:apply-templates mode="report"/>
    </div>
  </xsl:template>
  
  <xsl:template match="project | job" mode="report"/>
  
  <xsl:template match="job" mode="mention">
    <xsl:text> | </xsl:text>
    <xsl:apply-templates/>
  </xsl:template>
  
  <xsl:template match="role" mode="mention">
    <xsl:text> [</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>]</xsl:text>
  </xsl:template>
  
  <xsl:template match="day" mode="report">
    <xsl:if test="string()">
      <p class="day" style="margin-top: 0em; margin-bottom: 0.5em">
        <span class="date" style="font-family:sans-serif; font-size:80%; font-weight: bold">
          <xsl:value-of select="substring(format-date(xs:date(@date),'[FNn]'),1,3)"/>
          <xsl:text> </xsl:text>
          <xsl:value-of select="substring(format-date(xs:date(@date),'[MNn]'),1,3)"/>
          <xsl:text> </xsl:text>
          <xsl:value-of select="format-date(xs:date(@date),'[D]')"/>
          <xsl:text> </xsl:text>
        </span>
        <xsl:apply-templates mode="report"/>
      </p>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="time" mode="report">
    <span class="hours" style="font-weight: bold">
      <xsl:text> &#8210; </xsl:text>
      <xsl:apply-templates select="hours" mode="report"/>
    </span>
    <xsl:for-each select="start">
      <span class="duration" style="font-size:90%">
        <xsl:text> (from </xsl:text>
        <xsl:value-of select="."/>
        <xsl:text> to </xsl:text>
        <xsl:value-of select="following-sibling::end"/>
        <xsl:text>)</xsl:text>
      </span>
    </xsl:for-each>
  </xsl:template>
  
  <xsl:template match="start" mode="report">
    <span class="time" style="font-weight: bold; margin-top:0em; margin-bottom;0em">
      <xsl:text> (from </xsl:text>
      <xsl:apply-templates mode="report"/>
    </span>
  </xsl:template>
  
  <xsl:template match="end" mode="report">
    <span class="time" style="font-weight: bold; margin-top:0em; margin-bottom;0em">
      <xsl:text> to </xsl:text>
      <xsl:apply-templates mode="report"/>
      <xsl:text>)</xsl:text>
    </span>
  </xsl:template>
  
  <xsl:template match="work" mode="report">
    <span class="work">
      <xsl:apply-templates mode="report"/>
    </span>
  </xsl:template>
  
  <xsl:template match="role" mode="report"/>
  
  <xsl:template match="note" mode="report">
    <span class="note" style="font-style:italic">
      <xsl:text> &#8210; </xsl:text>
      <xsl:apply-templates mode="report"/>
    </span>
  </xsl:template>
  
  <xsl:template match="task" mode="report">
    <span class="note">
      <xsl:text> (</xsl:text>
      <xsl:apply-templates mode="report"/>
      <xsl:text>)</xsl:text>
    </span>
  </xsl:template>
  
  <xsl:template match="hours" mode="report">
    <xsl:apply-templates/>
    <xsl:text> hours</xsl:text>
  </xsl:template>
  
</xsl:stylesheet>
